; Fichero: PLL5.ASM
; Fecha de inicio: 25 de marzo de 2002
; ltima modificacin: 23 de octubre de 2003 (modificacin del tipo de flanco del PPS a ascendente)
; Probado en simulacin PC: 
; Probado en placas: 26 de marzo de 2002
; Comentarios a las pruebas:
; - Para la simulacin en PC, buscar los comentarios 'OJO! Simulacin PC' y seguir las
;   instrucciones.
; Mensajes al compilar: 
; - 6 del tipo: 'Register in operand not in bank 0.  Ensure that bank bits are correct.' 
;
; PLL5.ASM: Quinta versin del programa del PIC16C73A para los circuitos PLL del array del
;Vesubio. Est hecha a partir del PLL4.ASM con las siguientes modificaciones orientadas a
;solucionar el bug descrito en DIARIO.DOC del 25 de marzo de 2002 (primer prrafo):
; - Cuando se llega a la situacin de desajuste (diferencia PPS-DRDY mayor de 500us) se acta sobre 
;   el VCXO adems de activar el flag de pulso Sync. 
; - En situacin normal el control del VCXO se sigue llevando a cabo cada diez segundos. Sin
;   embargo, se incorpora una 'vigilancia de solapamiento' cada segundo, en la que se comprueba si
;   la seal DRDY va a solaparse con el PPS. En caso afirmativo, se disminuye la frecuencia del 
;   VCXO y se activa el flag de pulso /Sync. 
; - Se elimina el flag 'InicCtrol' y las funciones de ajuste rpido del VCXO. El ajuste se
;   realiza siempre de la forma ms suave posible (a travs de los bits <5:4> del registro
;   CCP2CON). Esta modificacin se debe a que en la situacin del 25 de marzo se observaban saltos
;   bruscos de la frecuencia del VCX0, quiz producidos por los ajustes rpidos.
;   
; Adems de estos, hay otros dos cambios que no tienen nada que ver con el bug:
; - El LED de Unlocked se enciende cuando se enva el pulso /Sync, y se apaga el segundo siguiente.
;   De esta forma, adems de ver si el PLL est desajustado, se puede calcular el nivel de 
;   desajuste en funcin del intervalo de tiempo que tarda el LED en encenderse.
; - Se quitan todas las lneas de cdigo que aparecan como comentarios en la versin anterior.
;
; Para realizar el control de la frecuencia del mdulo VCXO se programan los distintos mdulos del
;PIC16C73A en los siguientes modos: 
; - Mdulo CCP1 en modo captura. En realidad no se usa el mdulo de captura, pero s es necesario
;   programarlo as para usar la interrupcin del mdulo CCP1. Se podra haber usado la int.
;   externa INT/RB0, pero la placa del Vesubio est preparada para usar sta (ya que el PIC toma
;   la seal de PPS a travs del pin RC2).
; - Timer 1 en modo timer, para calcular el retardo entre el PPS y el flanco ascendente de la
;   primera seal DRDY. 
; - Mdulo CCP2 en modo PWM, con frecuencia de 9.766kHz, 'duty-cycle' inicial del 50% y resolucin
;   de 10 bits.
; - Se habilita una sola fuente de interrupcin:
;   - Interrupcin del mdulo CCP1, que se dispara por la seal externa de 1PPS (OJO con el tipo
;     de flanco -ascendente o descendente- habilitado para cada tipo de GPS). En la rutina de
;     servicio se espera hasta que se detecta el flanco ascendente de la primera seal DRDY, se 
;     calcula el retardo producido (mediante el Timer 1), se hacen los clculos correspondientes
;     para ver cmo hay que modificar la seal PWM y se modifica sta. 
;
; Para evitar que el ajuste del VCXO sea demasiado brusco y que la variable de control oscile en
;torno a la posicin de equilibrio, el control del VCXO se realiza de la siguiente forma:
; 1.- Cuando se recibe un PPS, se activa el Timer 1 (a partir de entonces el par de registros 
;     TMR1H:TMR1L se incrementarn con cada ciclo de instruccin).
; 2.- Se espera hasta que se recibe el primer flanco ascendente de DRDY, momento en el que se 
;     deshabilita el Timer 1 para que los registros TMR1H:TMR1L dejen de incrementarse.
; 3.- Se calcula la diferencia entre el valor en el que queremos centrar DRDY (200us = 500 ciclos
;     de instruccin) y el valor capturado de TMR1H y TMR1L. El resultado se expresa mediante las 
;     variables DifMSB, DifLSB (valor absoluto de la diferencia) y en los flags 'Signo'(signo de 
;     la diferencia) y 'DifCero'(se activa cuando la diferencia es cero). 
; 4.- Se compara el valor absoluto de la diferencia con el valor absoluto de la diferencia 
;     anterior (variables DifAntMSB y DifAntLSB). 
; 5.- Slo se acta sobre el VCXO si el valor absoluto actual es mayor o igual que el anterior.
;     Para ello se comprueban los flags 'Signo' y 'DifCero', para saber si hay que aumentar, 
;     disminuir o dejar igual el duty-cycle de la seal PWM.
; 6.- En caso de que el valor absoluto actual sea menor que el anterior no se acta sobre el VCXO, 
;     ya que es seal de que nos estamos acercando a la situacin correcta.
; 7.- En cualquier caso, antes de salir se pasa el valor de la diferencia actual a las variables 
;     DifAntMSB y DifAntLSB, para el prximo segundo.
; 8.- Cuando la frecuencia del VCXO se acerca a la correcta los valores calculados en el punto 4 
;     son muy pequeos, por lo que para minimizar el error en todo el proceso los clculos se 
;     realizan cada diez segundos. La principal causa de error en el clculo viene dada por la 
;     indeterminacin de 3 ciclos de instruccin introducida en el punto 2, ya que la deteccin 
;     de la seal DRDY se realiza mediante un bucle de espera. La otra causa de error es el 
;     tiempo de latencia de la interrupcin, que tiene un valor nominal de 3-4 ciclos de
;     instruccin. Para los clculos se ha tomado el valor 3.
; 9.- El ajuste de la frecuencia del VCXO se realiza ms rapidamente justo despus de enviar un 
;     pulso de sincronizacin, para compensar las desviaciones relativamente grandes que se dan 
;     despus de esa situacin. En este caso el ajuste del VCXO se realiza directamente a travs
;     del registro CCPR2L, ignorando el estado de los dos bits de control del mdulo PWM menos
;     significativos (bits <5:4> del registro CCP2CON).
;
; CONEXIONES (placa PLL3.SQM, definiciones en PLL.H):
;
;Puerto	 Pin	Descripcin
; RA0	  2	Salida LED 'Unlocked'
; RA1	  3	Salida monitorizacin de las interrupciones de PPS
; RB4	 25 	Entrada seal DRDY (Se usa esta por si interesa usar la 'interrrupt on change')
; RC0 	 11	Entrada Timer1 (frecuencia de salida del mdulo VCXO)
; RC1	 12	Salida mdulo CCP2 (control de la tensin de entrada del mdulo VCXO)
; RC2	 13	Entrada mdulo CCP1 (seal de 1PPS)
; RC3	 14	Salida seal /SYNC de los CADs de la placa A/D
; RC4	 15	Salida seal de Reset de la placa A/D
; RC5 	 16	Salida control de transmisin/recepcin del canal auxiliar RS-485
; RC6	 17	Salida del canal auxiliar RS-485
; RC7	 18	Entrada del canal auxiliar RS-485
;*************************************************************************************************

	LIST P=16C73A

        INCLUDE <C74_reg.h>	; El PIC16C73 tiene los mismos registros que el 16C74
	INCLUDE <PLL.H>
;
;*************************************************************************************************
; VARIABLES
;*************************************************************************************************
PosVar		EQU	0x20
MinMSB		EQU	PosVar		; Variables para el clculo de la diferencia entre el 
MinLSB		EQU	PosVar+.1	;retardo PPS-DRDY y el valor correcto de ese retardo 
SusMSB		EQU	PosVar+.2	;(200us = 500ciclos de instruccin)
SusLSB		EQU	PosVar+.3
DifMSB		EQU	PosVar+.4
DifLSB		EQU	PosVar+.5
DifAntMSB	EQU	PosVar+.6
DifAntLSB	EQU	PosVar+.7
Timer1H		EQU	PosVar+.8	; OJO! Simulacin PC, para simular distintos valores 
Timer1L		EQU	PosVar+.9	;capturados del timer 1
ContCheckDifs	EQU	PosVar+.10
ContReset	EQU	PosVar+.11	; Para la gestin del reset
Temp1		EQU	PosVar+.12
Temp2		EQU	PosVar+.13
Temp3		EQU	PosVar+.14
Flag		EQU	PosVar+.15	; Para definir indicadores (flags)
HayDRDY		EQU	0		; Bit 0 de Flag: Los AD7710s han empezado a adquirir datos
PulsSync	EQU	1		; Bit 1 de Flag: La siguiente int. de PPS debe generar un 
					;pulso de sincronismo para los AD7710s
Signo		EQU	2		; Bit 2 de Flag: Signo de la diferencia 500-Tmr1 (0 = 
					;positivo, 1 = negativo)
DifCero		EQU	3		; Bit 3 de Flag: Se activa cuando la diferencia 500-Tmr1 = 0
PPS		EQU	4		; Bit 5 de Flag: Se ha recibido un PPS
;
;*************************************************************************************************
; FUNCIONES 
;*************************************************************************************************
		ORG	0x030		
;-------------------------------------------------------------------------------------------------
; Inicializacion.- Inicializacin de los pines I/O, variables, timers, mdulos CCP e 
;interrupciones.
;-------------------------------------------------------------------------------------------------
Inicializacion
		; 1. Inicializacin de pines I/O:
		BSF	STATUS, RP0	; Banco 1
		MOVLW	0x07		; Configura los bits del puerto A como I/O digitales
		MOVWF	ADCON1
		BCF	STATUS, RP0	; Banco 0
		CLRF	PORTA		; Inicializa PORTA, PORTB y PORTC limpiando las salidas
		CLRF	PORTB
		CLRF	PORTC
		BSF	STATUS, RP0	; Banco 1
		MOVLW	0x04		; PORTA: 0x04 = 00000100b: pin 2 entrada, el resto salidas
		MOVWF	TRISA		
		MOVLW	0x10		; PORTB: 0x10 = 00010000b: pin 4 entrada, el resto salidas
		MOVWF	TRISB		
		MOVLW	0x85		; PORTC: 0x85 = 10000101b: pines 0, 2 y 7 entradas, el 
		MOVWF	TRISC		;resto salidas
		BCF	STATUS, RP0	; Banco 0

		; 2. Timer 1 en modo timer:
		CLRF	T1CON		; Conf. del Timer1: 0x00: Timer1 deshabilitado (de momento),
					;modo timer, oscilador deshabilitado, prescaler a 1:1

		; 3. Mdulo CCP2 en modo PWM:
		BSF	STATUS, RP0	; Banco 1
		MOVLW	0xFF    	; Fija el periodo de PWM a 9.766kHz (con cristal de 10MHz)
		MOVWF	PR2		;o 3.906kHz (con cristal de 4MHz)
		BCF	STATUS, RP0	; Banco 0
		MOVLW	0x80		; Inicializa el duty-cycle del PWM (CCPR2L + CCP2CON<5:4>)
		MOVWF	CCPR2L		;al 50% y el mdulo CCP2 a modo PWM (haciendo CCP2CON<3:0>
		MOVLW	0x0C		; = 11xx)
		MOVWF	CCP2CON		
		MOVLW	0x04		; Habilita el Timer2 con prescaler y postscaler a 1:1
		MOVWF	T2CON		

		; 4. Inicializacin de las interrupciones y mdulo CCP1 en modo captura:
		CLRF	PIR1		; Pone a 0 todos los flags de interrupcin
		CLRF	PIR2		
		; OJO! FLANCO DESCENDENTE PARA LAS PRUEBAS CON EL CHAMPIN (Trimble), ASCENDENTE PARA
		;EL Garmin HVS-35
		MOVLW	0x05		; Mdulo CCP1 en modo captura, interrupcion con cada flanco
		MOVWF	CCP1CON		;ASCENDENTE del pin RC2/CCP1
		BSF	INTCON, PEIE	; Habilita interrupciones perifericos
		BSF	STATUS, RP0	; Banco 1
		BSF	PIE1, CCP1IE	; Habilita la interrupcion de CCP1
		BCF	STATUS, RP0	; Banco 0 (se queda habilitado a la salida de la funcin)
		BSF	INTCON, GIE	; Habilita las interrupciones

		; 5. Inicializacin de variables:
		CLRF	Flag		; Inicializa los flags
		CLRF	MinMSB		; Limpia todas las variables para el clculo de la 
		CLRF	MinLSB		;diferencia
		CLRF	SusMSB		
		CLRF	SusLSB
		CLRF	DifMSB
		CLRF	DifLSB
		CLRF	DifAntMSB
		CLRF	DifAntLSB
		MOVLW	.10		;Inicializa el contador de reset a 10 (para que se produzca
		MOVWF	ContReset	;un reset de la placa A/D despus de 10 segs. sin recibir PPS)
		MOVLW	.10		; La primera captura del Timer 1 y modificacin del VCXO
		MOVWF	ContCheckDifs	;se lleva a cabo a los diez segundos.
		BSF	PORTC, Sync	; Los pulsos Sync son negativos (OJO! Si no se hace esto,
					;no se permite al PIC del mdulo de adquisicin inicializar
					;los CADs).
		; 6. Antes de permitir el control del VCXO se espera a detectar dos DRDYs. De 
		;    esta forma nos aseguramos de que el control del VCXO no empieza antes que la
		;    adquisicin de datos:
WaitDRDY_0A
		BTFSC	PORTB, DRDY
		GOTO	WaitDRDY_0A
WaitDRDY_1A
		BTFSS	PORTB, DRDY
		GOTO	WaitDRDY_1A
WaitDRDY_0B
		BTFSC	PORTB, DRDY
		GOTO	WaitDRDY_0B
WaitDRDY_1B
		BTFSS	PORTB, DRDY
		GOTO	WaitDRDY_1B
WaitDRDY_0C
		BTFSC	PORTB, DRDY
		GOTO	WaitDRDY_0C
		
		BSF	Flag, HayDRDY	; Activa el Flag de DRDY, para que la ISR empiece a 
		BSF	Flag, PulsSync	;realizar las tareas que tiene asignadas, y el Flag de 
					;PulsSync, para que la ISR del siguiente PPS genere un
					;pulso de sincronismo de los CADs.
		RETURN			
;
;-------------------------------------------------------------------------------------------------
; GestIntPPS.- Funcin de gestin de la interrupcin de CCP1, que es disparada por la seal de un 
;pulso por segundo (PPS). En primer lugar programa el Timer 1 en modo timer, para ver el tiempo que
;tarda en activarse la seal DRDY de los CADs. Luego comprueba se hay que enviar un pulso de 
;sincronizacin a los CADs y, en caso afirmativo, lo hace. Despus entra en un bucle de espera,
;hasta que detecta que la seal DRDY de los CADs se ha activado. En funcin del tiempo que ha
;tardado, modifica la tensin de entrada del VCXO. Para que el ajuste sea suave y minimizar errores
;en el clculo de la diferencia, toda esta gestin del VCXO slo se lleva a cabo una vez cada diez
;segundos.
;-------------------------------------------------------------------------------------------------
GestIntPPS				; Banco 0 habilitado antes
		BCF	PIR1, CCP1IF	; Limpia el flag de interrupcin
		BSF	Flag, PPS	; Para la gestin del reset del programa principal
		BTFSS	Flag, HayDRDY	; Slo ejecuta el resto de la funcin si se ha detectado 
		RETURN			;que los CADs han empezado a adquirir datos (hay seal DRDY)
		BCF	T1CON, TMR1ON	; Deshabilita el timer1
		CLRF	TMR1H		; Inicializa los registros del timer1 a 18, que es el
		MOVLW	.18		;nmero de ciclos de instruccin que se pierden desde que
		MOVWF	TMR1L		;se detecta el pulso de segundo hasta que se habilita el 
					;timer1 (contando con 3 ciclos antes de saltar a la ISR)
		BSF	T1CON, TMR1ON	; Habilita el timer1
		
		BCF	PORTA, LEDSync	; Apaga, si estaba encendido, el LED de pulso /Sync
		BTFSS	Flag, PulsSync	; Genera el pulso /Sync (si el flag correspondiente est
		GOTO	ChkDRDY		;activado)
		BCF	Flag, PulsSync	; Limpia el flag de enviar pulso /Sync
		BSF	PORTA, LEDSync	
		BCF	PORTC, Sync	; Genera un pulso de sincronizacin negativo de 4us de 
		NOP			;duracin (con cristal de 10MHz).
		NOP			
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		BSF	PORTC, Sync
		GOTO	SigueInt						

		; Cada segundo comprueba si DRDY se va a cruzar con PPS. En caso afirmativo activa
		;el flag de PulsSync, deshabilita el timer 1, inicializa el contador de segundos y
		;las variables de diferencia y baja la frecuencia del VCXO. 
ChkDRDY
		BTFSS	PORTB, DRDY
		GOTO	SigueInt
		BSF	Flag, PulsSync
		BCF	T1CON, TMR1ON	; Deshabilita el timer 1
		MOVLW	.10
		MOVWF	ContCheckDifs
		CLRF	DifMSB
		CLRF	DifLSB
		GOTO	BajarPWM

SigueInt
		DECFSZ	ContCheckDifs, F; Las diferencias se calculan slo cada 10 segundos
		RETURN
		MOVLW	.10
		MOVWF	ContCheckDifs

EsperaDRDY_1
		BTFSS	PORTB, DRDY
		GOTO	EsperaDRDY_1
		BCF	T1CON, TMR1ON	; Deshabilita el timer 1

		CALL	Diferencia	; Calcula la diferencia entre el valor correcto (200us = 
					;= 500 ciclos de instruccin) y el capturado en TMR1H:TMR1L.
		CALL	ChkSync		; Comprueba si hay que resincronizar los conversores. Esto
		SUBLW	.1		;se hace si la diferencia es mayor de 300us (distancia
		BTFSC	STATUS, Z	;PPS-DRDY mayor de 500us). En caso de que sea as se activa
		BSF	Flag, PulsSync	;el Flag de pulso /Sync para generar un pulso en el
					;siguiente PPS, pero no se reinicializa el PWM porque se
					;podra entrar en un bucle Sync-Reinicializar PWM.
		MOVF	DifAntMSB, W	; Compara el valor absoluto de la diferencia con el de 
		SUBWF	DifMSB, W	;la diferencia anterior.
		BTFSC 	STATUS, Z	; Si los dos MSBs son iguales, compara los LSBs
		GOTO	CheckLSBs
		BTFSS	STATUS, C	; Si la diferencia actual es menor que la anterior, no se
		GOTO	FinGestIntPPS	;hace nada (nos estamos acercando al valor correcto)
		GOTO	CheckSigno	; Si la diferencia actual es mayor o igual que la anterior
					;se comprueba el signo de la actual para ver si hay que 
					;subir o bajar el PWM
CheckLSBs
		MOVF	DifAntLSB, W	; Compara el valor absoluto de la diferencia con el de 
		SUBWF	DifLSB, W	;la diferencia anterior.
		BTFSC	STATUS, C
		GOTO	CheckSigno	; Si la diferencia actual es mayor o igual que la anterior
					;se comprueba el signo de la actual para ver si hay que 
					;subir o bajar el PWM
		GOTO	FinGestIntPPS	; Si la diferencia actual es menor que la anterior, no se
					;modifica el VCXO (nos estamos acercando al valor correcto).
CheckSigno
		BTFSC	Flag, DifCero	; Si la diferencia actual es cero, no se hace nada (estamos
		GOTO	FinGestIntPPS	;en la frecuencia buena y con el retardo bueno!)
		BTFSC	Flag, Signo
		GOTO	SubirPWM
BajarPWM
		CALL	BajaPWM		
		GOTO	FinGestIntPPS	
SubirPWM				; Signo negativo => 500<Tmr1 => Hay que aumentar PWM
		CALL	SubePWM		
FinGestIntPPS
		MOVF	DifMSB, W	; Antes de salir, pasamos la diferencia recin calculada
		MOVWF	DifAntMSB	;a las variables de diferencia anterior, para la prxima
		MOVF	DifLSB, W	;vez.
		MOVWF	DifAntLSB
		RETURN
;
;-------------------------------------------------------------------------------------------------
; ChkSync.- Comprueba si hay que resincronizar los conversores. Esto se hace si la diferencia 
;entre la posicin buena de la seal DRDY y la real es mayor de 300us (que equivale a una distancia
;PPS-DRDY mayor de 500us). La diferencia entre la posicin buena y la real se ha calculado antes en
;la funcin 'Diferencia', y est en las variables DifLSB y DifMSB.
; Devuelve en W: 0 si el valor absoluto de la diferencia entre la posicin buena de DRDY y la 
;medida es menor que 300us (750 ciclos de instruccin = 0x2EE), 1 si es mayor.
;-------------------------------------------------------------------------------------------------
ChkSync
		MOVLW	0x02
		SUBWF	DifMSB, W
		BTFSS	STATUS, C
		RETLW	.0		; Si el MSB de la diferencia es menor que 2, est bien
		BTFSS	STATUS, Z
		RETLW 	.1		; Si el MSB de la diferencia es mayor que 2, est desajustado
		MOVLW	0xEE		; Si es igual, hay que comprobar el LSB de la diferencia
		SUBWF	DifLSB, W
		BTFSS	STATUS, C
		RETLW	.0		; Si el MSB es igual a 2 y el LSB es menor que 0xEE, est bien
		RETLW	.1		; Si el MSB es igual a 2 y el LSB es mayor o igual que 0xEE,
					;est desajustado
;
;-------------------------------------------------------------------------------------------------
; SubePWM.- Sube el duty-cycle de la seal PWM, a travs de los bits CCP2X:CCP2Y de CCP2CON (LSbs)
;y del registro CCPR2L. 
;-------------------------------------------------------------------------------------------------
SubePWM
		BTFSC	CCP2CON, CCP2Y	; Comprueba el estado del LSb del PWM (bit CCP2Y)
		GOTO	IncCCP2X	; Si es 1, hay que incrementar el siguiente bit
		BSF	CCP2CON, CCP2Y	; Si es 0 basta con ponerlo a 1, no es necesario
		RETURN			;incrementar el registro CCPR2L
IncCCP2X
		BCF	CCP2CON, CCP2Y	; Pone el LSb a 0
		BTFSC	CCP2CON, CCP2X	; Comprueba el estado del 2 bit del PWM (bit CCP2X)
		GOTO	IncCCPR2L	; Si el 2 bit es 1, hay que incrementar el reg. CCPR2L
		BSF	CCP2CON, CCP2X	; Si el 2 bit es 0 basta con ponerlo a 1, no es
		RETURN			;necesario incrementar el registro CCPR2L
IncCCPR2L
		BCF	CCP2CON, CCP2X	; CCP2X:CCP2Y = 11, por lo que se ponen los dos a 0 y se
		INCF	CCPR2L, F	;incrementa el registro CCPR2L
		RETURN
;
;-------------------------------------------------------------------------------------------------
; BajaPWM.- Baja el duty-cycle de la seal PWM, a travs de los bits CCP2X:CCP2Y de CCP2CON (LSbs)
;y del registro CCPR2L. 
;-------------------------------------------------------------------------------------------------
BajaPWM
		BTFSS	CCP2CON, CCP2Y	; Comprueba el estado del LSb del PWM (bit CCP2Y)
		GOTO	DecCCP2X	; Si es 0, hay que decrementar el siguiente bit
		BCF	CCP2CON, CCP2Y	; Si es 1 basta con ponerlo a 0, no es necesario
		RETURN			;decrementar el registro CCPR2L
DecCCP2X
		BSF	CCP2CON, CCP2Y	; Pone el LSb a 1
		BTFSS	CCP2CON, CCP2X	; Comprueba el estado del 2 bit del PWM (bit CCP2X)
		GOTO	DecCCPR2L	; Si el 2 bit es 0, hay que decrementar el reg. CCPR2L
		BCF	CCP2CON, CCP2X	; Si el 2 bit es 1 basta con ponerlo a 0, no es 
		RETURN			;necesario decrementar el registro CCPR2L
DecCCPR2L
		BSF	CCP2CON, CCP2X	; CCP2X:CCP2Y = 00, por lo que se ponen los dos a 1 y se
		DECF	CCPR2L, F	;decrementa el registro CCPR2L
		RETURN
;
;-------------------------------------------------------------------------------------------------
; Diferencia.- Calcula la diferencia entre el retardo correcto (entre las seales de PPS y DRDY) y
;el medido. El retardo medido es el capturado en el par de registros del timer 1 (TMR1H:TMR1L). El
;retardo correcto son 500 ciclos de instruccin (200us).
; Para el clculo usa las variables MinMSB, MinLSB, SusMSB, SusLSB, DifMSB, DifLSB (Minuendo, 
;sustraendo y diferencia). Como salida da el valor absoluto de la diferencia en las variables 
;DifMSB y DifLSB y el signo en el flag de signo: 0 si la diferencia es positiva o cero (es decir, 
;el valor medido es menor o igual que el correcto) y 1 si es negativa (valor medido mayor que el
;correcto). 
;-------------------------------------------------------------------------------------------------
Diferencia
		;MOVLW	0x01		; OJO! Simulacin PC de los registros del Tmr1 (quitar
		;MOVWF	Timer1H		;los comentarios)
		;MOVLW	0xF4
		;MOVWF	Timer1L
		;MOVF	Timer1H, W	; El valor capturado del timer 1 se mete en el sustraendo
		;MOVWF	SusMSB
		;MOVF	Timer1L, W
		;MOVWF	SusLSB

		MOVLW	0x01		; La cuenta correcta (500ciclos = 0x1F4) se mete en el
		MOVWF	MinMSB		;minuendo
		MOVLW	0xF4
		MOVWF	MinLSB
					; OJO! Simulacin PC de los regs. del Tmr1 (comentar)
		MOVF	TMR1H, W	; El valor capturado del timer 1 se mete en el sustraendo
		MOVWF	SusMSB
		MOVF	TMR1L, W
		MOVWF	SusLSB

		BCF	Flag, DifCero	; Por defecto, la diferencia es distinta de cero y de 
		BCF	Flag, Signo	;signo positivo (flags de cero y de signo a 0)
Resta
		MOVF	SusLSB, W
		SUBWF	MinLSB, W
		MOVWF	DifLSB
		BTFSC	STATUS, C	; Si el resultado es negativo, se le suma 1 al MSB del 
		GOTO	SigueResta	;sustraendo
		MOVLW	.1
		ADDWF	SusMSB, F
SigueResta		
		MOVF	SusMSB, W
		SUBWF	MinMSB, W
		MOVWF	DifMSB
		BTFSS	STATUS, C
		GOTO	DifNeg
		BTFSS	STATUS, Z	
		RETURN			; Antes de salir, comprueba si la diferencia es 0 (tanto
		MOVF	DifLSB, F	;el MSB recin calculado como el LSB calculado antes). En
		BTFSS	STATUS, Z	;caso afirmativo, activa el flag correspondiente.
		RETURN	
		BSF	Flag, DifCero	; Si la diferencia es 0, se activa el flag DifCero antes 
		RETURN			;de salir
DifNeg
		BSF	Flag, Signo	; Si llega aqu, la diferencia es negativa => se pone a 1
					;el flag de signo y se hace la resta al	revs (valor 
					;capturado del timer 1 - 500)
		MOVF	TMR1H, W	; OJO! Simulacin PC de los regs. del Tmr1 (comentar)
		MOVWF	MinMSB		
		MOVF	TMR1L, W
		MOVWF	MinLSB

		;MOVF	Timer1H, W	; OJO! Simulacin PC de los regs. del Tmr1 (quitar
		;MOVWF	MinMSB		;comentarios).
		;MOVF	Timer1L, W
		;MOVWF	MinLSB

		MOVLW	0x01		
		MOVWF	SusMSB
		MOVLW	0xF4
		MOVWF	SusLSB
		
		GOTO	Resta
;
;-------------------------------------------------------------------------------------------------
; GestReset.- Gestiona el reseteo del PIC de la placa de conversores. Cuando pasan diez segundos
;sin recibirse pulsos de segundo se manda un pulso de reset a la placa de conversores y se salta
;al inicio del programa.
; Devuelve en W: 1 si se ha cumplido el time out y se ha enviado un pulso de reset a la placa de 
;conversores, 0 en caso contrario. 
;-------------------------------------------------------------------------------------------------
GestReset
		CALL	Retardo1s
		BTFSS	Flag, PPS
		GOTO	DecCont
		BCF	Flag, PPS		; Si se ha recibido un PPS se reinicializa el 
		MOVLW	.10			;contador de reset
		MOVWF	ContReset
		RETLW	.0
DecCont
		DECFSZ	ContReset, F		; Si el contador de reset se hace cero se enva un
		RETLW	.0			;pulso de reset a la placa de A/D
		BSF	PORTC, ResetAD
		CALL	Retardo1s
		BCF	PORTC, ResetAD
		RETLW	.1
;
;-------------------------------------------------------------------------------------------------
; Retardo1ms.- Genera un retardo de aproximadamente 1ms (con cristal de 10MHz). 
; Nmero de ciclos = 1ms/(0.4us/ciclo) = 2500ciclos. Con 10 ciclos por bucle:
; Nmero de bucles = 2500ciclos/(10ciclos/bucle) = 250bucles
;-------------------------------------------------------------------------------------------------
Retardo1ms
		MOVLW	.250
		MOVWF	Temp1
BucleRet
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		DECFSZ	Temp1, F
		GOTO	BucleRet
		RETURN
;
;-------------------------------------------------------------------------------------------------
; Retardo1s.- Genera un retardo de aproximadamente 1s (con cristal de 10MHz).
; Hay que llamar a la funcin Retardo1ms 1000veces. OJO! No se puede usar la variable Temp1 como
;contador, porque es la que usa Retardo1ms.
;-------------------------------------------------------------------------------------------------
Retardo1s
		MOVLW	.4
		MOVWF	Temp2
InicTemp3
		MOVLW	.250
		MOVWF	Temp3
MasRet1ms
		CALL	Retardo1ms
		DECFSZ	Temp3, F
		GOTO	MasRet1ms
		DECFSZ	Temp2, F
		GOTO	InicTemp3
		RETURN	
;
;*************************************************************************************************
; RUTINA DE SERVICIO DE INTERRUPCIN
;*************************************************************************************************
		ORG 	ISR_V		; Vector de interrupcion
		BCF	STATUS, RP0	; Banco 0
		BTFSS	PIR1, CCP1IF	; Comprueba si es la interrupcin de CCP1 (PPS)
		GOTO	FinISR
		BSF	PORTA, LEDPPS	; Enciende el LED de PPS
		CALL	GestIntPPS
		CALL	Retardo1ms	; Retardo de 2ms, para que pueda verse el LED de PPS
		CALL	Retardo1ms	
		BCF	PORTA, LEDPPS	; Apaga el LED de PPS
FinISR
		BCF	INTCON, RBIF	; Limpia los flags de todas las interrupciones salvo la de
		BCF 	INTCON, INTF	;CCP1, que se limpia en la funcin 'GestIntPPS'
		BCF	INTCON, T0IF	
		MOVLW	0x04		
		ANDWF	PIR1, F		
		CLRF	PIR2		
		RETFIE			
;
;*************************************************************************************************
; PROGRAMA PRINCIPAL
;*************************************************************************************************
		ORG	RESET_V
		GOTO	Inicio
;
		ORG	0x750
Inicio
		; 1. Inicializacin de puertos de I/O y variables y programacin de interrupciones,
		;    timers y mdulos CCP. El programa no sale de esta funcin hasta que detecta
		;    dos seales DRDY.
		CALL	Inicializacion

		; 2. Bucle principal, en el que slo se comprueba el tiempo que ha transcurrido sin
		;    PPS para ver si hay que mandar un carcter de reset a la placa de adquisicin. 
		;    El control del VCXO se realiza en la ISR (funcin 'GestIntPPS').
bucle
		CALL	GestReset
		SUBLW	.1		; Si GestReset devuelve un 1 hay un comando de reset del 
		BTFSC	STATUS, Z	;sistema, por lo que se salta al inicio del programa
		GOTO	Inicio		;despus de enviar un pulso de reset a la placa de los 
		GOTO 	bucle		;conversores.
		END			


